`timescale 1 ns / 1 ps

module uart_tx(
	input clk, //24MHz
	input rst_n,
	
	input [7:0] buffer,
	input buffer_clk,
	
	output reg [3:0] buffer_addr,
	
	output reg tx
);

reg [3:0] state;
parameter IDLE = 4'b0001, DATA = 4'b0100;

reg [7:0] baud_counter; // 208: 0~207

reg [3:0] bit_counter;
wire [9:0] bits;
assign bits = {1'b1, buffer, 1'b0};

always @(posedge clk or negedge rst_n)
	if (!rst_n) begin
		state <= IDLE;
		buffer_addr <= 4'd0;
		baud_counter <= 8'd0;
		bit_counter <= 4'd0;
		tx <= 1'b1;
	end
	else begin
		
		case (state)
		IDLE: begin
			buffer_addr <= 4'd0;
			baud_counter <= 8'd0;
			bit_counter <= 4'd0;
			tx <= 1'b1;
			if (buffer_clk)
				state <= DATA;
		end
		
		DATA: begin
			baud_counter <= baud_counter + 1'b1;
			tx <= bits[bit_counter];
			if (baud_counter == 8'd207) begin
				baud_counter <= 8'd0;
				bit_counter <= bit_counter + 1'b1;
				if (bit_counter == 4'd9) begin
					bit_counter <= 4'd0;
					buffer_addr <= buffer_addr + 1'b1;
					if (buffer_addr == 4'd9) begin
						buffer_addr <= 4'd0;
						state <= IDLE;
					end
				end
			end
		end
		
		default: begin
			state <= IDLE;
		end
		
		endcase
	end

endmodule
