`timescale 1 ns / 1 ps

module usb_rx(
	input clk, //24MHz
	input rst_n,
	
	input dm,
	input dp,
	
	output data,
	output data_empty,
	output data_clk
);

reg data_reg;
reg data_empty_reg;
reg data_clk_reg;

reg dp_reg;
reg [3:0] count; // 0~15
reg idle_reg;

always @(posedge clk or negedge rst_n)
	if (!rst_n) begin
		count <= 4'd0;
		data_reg <= 1'b0;
		data_empty_reg <= 1'b0;
		data_clk_reg <= 1'b0;
		idle_reg <= 1'b1;
	end
	else begin
		dp_reg <= dp;
		//count <= count + 1'b1;
		
		if (dp_reg != dp) begin
			count <= 4'd0;
			idle_reg <= 1'b0;
		end
		else begin
			count <= count + 1'b1;
			if (idle_reg == 1'b0 && count == 4'd8) begin
				if (dp == 1'b0 && dm == 1'b0) begin
					idle_reg <= 1'b1;
					data_empty_reg <= 1'b1;
					data_clk_reg <= 1'b1;
				end
				else begin
					data_reg <= dp;
					data_clk_reg <= 1'b1;
				end
			end
			else begin
				data_empty_reg <= 1'b0;
				data_clk_reg <= 1'b0;
			end
		end
	end

reg decode_reg;
reg decode_empty_reg;
reg decode_clk_reg;

assign data = decode_reg;
assign data_empty = decode_empty_reg;
assign data_clk = decode_clk_reg;

reg last_data;
reg [2:0] decode_count; //count for 1

always @(posedge clk or negedge rst_n)
	if (!rst_n) begin
		last_data <= 1'b0;
		decode_reg <= 1'b0;
		decode_empty_reg <= 1'b0;
		decode_clk_reg <= 1'b0;
		decode_count <= 3'd0;
	end
	else if (data_clk_reg == 1'b1) begin
		
		if (data_empty_reg == 1'b1) begin
			last_data <= 1'b0;
			decode_empty_reg <= 1'b1;
			decode_clk_reg <= 1'b1;
			decode_count <= 3'd0;
		end
		else begin
			if (data_reg != last_data) begin
			
				if (decode_count != 3'd6) begin
					decode_reg <= 1'b0;
					decode_clk_reg <= 1'b1;
				end
				decode_count <= 3'd0;
			end
			else begin
				decode_count <= decode_count + 1'b1;
				decode_reg <= 1'b1;
				decode_clk_reg <= 1'b1;
			end
			last_data <= data_reg;
		end
	end
	else begin
		decode_empty_reg <= 1'b0;
		decode_clk_reg <= 1'b0;
	end
	
endmodule
